@@ -32,16 +32,17 @@ namespace pv {
 namespace data {
 namespace decode {
 
-Annotation::Annotation(const srd_proto_data *const pdata) :
+Annotation::Annotation(const srd_proto_data *const pdata, const Row *row) :
 	start_sample_(pdata->start_sample),
-	end_sample_(pdata->end_sample)
+	end_sample_(pdata->end_sample),
+	row_(row)
 {
 	assert(pdata);
 	const srd_proto_data_annotation *const pda =
 		(const srd_proto_data_annotation*)pdata->data;
 	assert(pda);
 
-	format_ = pda->ann_class;
+	ann_class_ = (Class)(pda->ann_class);
 
 	const char *const *annotations = (char**)pda->ann_text;
 	while (*annotations) {
@@ -60,9 +61,9 @@ uint64_t Annotation::end_sample() const
 	return end_sample_;
 }
 
-int Annotation::format() const
+Annotation::Class Annotation::ann_class() const
 {
-	return format_;
+	return ann_class_;
 }
 
 const vector<QString>& Annotation::annotations() const
@@ -70,6 +71,16 @@ const vector<QString>& Annotation::annotations() const
 	return annotations_;
 }
 
+const Row* Annotation::row() const
+{
+	return row_;
+}
+
+bool Annotation::operator<(const Annotation &other) const
+{
+	return (start_sample_ < other.start_sample_);
+}
+
 } // namespace decode
 } // namespace data
 } // namespace pv
