@@ -30,12 +30,14 @@
 #include <libsigrokcxx/libsigrokcxx.hpp>
 
 #include <QApplication>
+#include <QDebug>
 #include <QObject>
 #include <QProgressDialog>
 
 #include <boost/filesystem.hpp>
 
 #include <pv/devices/hardwaredevice.hpp>
+#include <pv/util.hpp>
 
 using std::bind;
 using std::list;
@@ -45,6 +47,7 @@ using std::placeholders::_2;
 using std::shared_ptr;
 using std::string;
 using std::unique_ptr;
+using std::vector;
 
 using Glib::VariantBase;
 
@@ -54,40 +57,96 @@ using sigrok::Driver;
 
 namespace pv {
 
-DeviceManager::DeviceManager(shared_ptr<Context> context) :
+DeviceManager::DeviceManager(shared_ptr<Context> context,
+	std::string driver, bool do_scan) :
 	context_(context)
 {
 	unique_ptr<QProgressDialog> progress(new QProgressDialog("",
-		QObject::tr("Cancel"), 0, context->drivers().size()));
+		QObject::tr("Cancel"), 0, context->drivers().size() + 1));
 	progress->setWindowModality(Qt::WindowModal);
 	progress->setMinimumDuration(1);  // To show the dialog immediately
 
 	int entry_num = 1;
 
-	for (auto entry : context->drivers()) {
+	/*
+	 * Check the presence of an optional user spec for device scans.
+	 * Determine the driver name and options (in generic format) when
+	 * applicable.
+	 */
+	std::string user_name;
+	vector<std::string> user_opts;
+	if (!driver.empty()) {
+		user_opts = pv::util::split_string(driver, ":");
+		user_name = user_opts.front();
+		user_opts.erase(user_opts.begin());
+	}
+
+	/*
+	 * Scan for devices. No specific options apply here, this is
+	 * best effort auto detection.
+	 */
+	for (auto& entry : context->drivers()) {
+		if (!do_scan)
+			break;
+
+		// Skip drivers we won't scan anyway
+		if (!driver_supported(entry.second))
+			continue;
+
 		progress->setLabelText(QObject::tr("Scanning for %1...")
 			.arg(QString::fromStdString(entry.first)));
 
-		/**
-		 * We currently only support devices that can deliver
-		 * samples at a fixed samplerate i.e. oscilloscopes and
-		 * logic analysers.
-		 * @todo Add support for non-monotonic devices i.e. DMMs
-		 * and sensors.
-		 */
-		const auto keys = (entry.second)->config_keys();
-
-		bool supported_device = keys.count(ConfigKey::LOGIC_ANALYZER) |
-			keys.count(ConfigKey::OSCILLOSCOPE);
-
-		if (supported_device)
-			driver_scan(entry.second, map<const ConfigKey *, VariantBase>());
+		if (entry.first == user_name)
+			continue;
+		driver_scan(entry.second, map<const ConfigKey *, VariantBase>());
 
 		progress->setValue(entry_num++);
 		QApplication::processEvents();
 		if (progress->wasCanceled())
 			break;
 	}
+
+	/*
+	 * Optionally run another scan with potentially more specific
+	 * options when requested by the user. This is motivated by
+	 * several different uses: It can find devices that are not
+	 * covered by the above auto detection (UART, TCP). It can
+	 * prefer one out of multiple found devices, and have this
+	 * device pre-selected for new sessions upon user's request.
+	 */
+	user_spec_device_.reset();
+	if (!driver.empty()) {
+		shared_ptr<sigrok::Driver> scan_drv;
+		map<const ConfigKey *, VariantBase> scan_opts;
+
+		/*
+		 * Lookup the device driver name.
+		 */
+		map<string, shared_ptr<Driver>> drivers = context->drivers();
+		auto entry = drivers.find(user_name);
+		scan_drv = (entry != drivers.end()) ? entry->second : nullptr;
+
+		/*
+		 * Convert generic string representation of options
+		 * to the driver specific data types.
+		 */
+		if (scan_drv && !user_opts.empty()) {
+			auto drv_opts = scan_drv->scan_options();
+			scan_opts = drive_scan_options(user_opts, drv_opts);
+		}
+
+		/*
+		 * Run another scan for the specified driver, passing
+		 * user provided scan options this time.
+		 */
+		list< shared_ptr<devices::HardwareDevice> > found;
+		if (scan_drv) {
+			found = driver_scan(scan_drv, scan_opts);
+			if (!found.empty())
+				user_spec_device_ = found.front();
+		}
+	}
+	progress->setValue(entry_num++);
 }
 
 const shared_ptr<sigrok::Context>& DeviceManager::context() const
@@ -106,6 +165,78 @@ DeviceManager::devices() const
 	return devices_;
 }
 
+/**
+ * Get the device that was detected with user provided scan options.
+ */
+shared_ptr<devices::HardwareDevice>
+DeviceManager::user_spec_device() const
+{
+	return user_spec_device_;
+}
+
+/**
+ * Convert generic options to data types that are specific to Driver::scan().
+ *
+ * @param[in] user_spec Vector of tokenized words, string format.
+ * @param[in] driver_opts Driver's scan options, result of Driver::scan_options().
+ *
+ * @return Map of options suitable for Driver::scan().
+ */
+map<const ConfigKey *, Glib::VariantBase>
+DeviceManager::drive_scan_options(vector<string> user_spec,
+	set<const ConfigKey *> driver_opts)
+{
+	map<const ConfigKey *, Glib::VariantBase> result;
+
+	for (auto& entry : user_spec) {
+		/*
+		 * Split key=value specs. Accept entries without separator
+		 * (for simplified boolean specifications).
+		 */
+		string key, val;
+		size_t pos = entry.find("=");
+		if (pos == std::string::npos) {
+			key = entry;
+			val = "";
+		} else {
+			key = entry.substr(0, pos);
+			val = entry.substr(pos + 1);
+		}
+
+		/*
+		 * Skip user specifications that are not a member of the
+		 * driver's set of supported options. Have the text format
+		 * input spec converted to the required driver specific type.
+		 */
+		const ConfigKey *cfg;
+		try {
+			cfg = ConfigKey::get_by_identifier(key);
+			if (!cfg)
+				continue;
+			if (driver_opts.find(cfg) == driver_opts.end())
+				continue;
+		} catch (...) {
+			continue;
+		}
+		result[cfg] = cfg->parse_string(val);
+	}
+
+	return result;
+}
+
+bool DeviceManager::driver_supported(shared_ptr<Driver> driver) const
+{
+	/*
+	 * We currently only support devices that can deliver samples at
+	 * a fixed samplerate (i.e. oscilloscopes and logic analysers).
+	 *
+	 * @todo Add support for non-monotonic devices (DMMs, sensors, etc).
+	 */
+	const auto keys = driver->config_keys();
+
+	return keys.count(ConfigKey::LOGIC_ANALYZER) | keys.count(ConfigKey::OSCILLOSCOPE);
+}
+
 list< shared_ptr<devices::HardwareDevice> >
 DeviceManager::driver_scan(
 	shared_ptr<Driver> driver, map<const ConfigKey *, VariantBase> drvopts)
@@ -114,27 +245,36 @@ DeviceManager::driver_scan(
 
 	assert(driver);
 
+	if (!driver_supported(driver))
+		return driver_devices;
+
 	// Remove any device instances from this driver from the device
 	// list. They will not be valid after the scan.
 	devices_.remove_if([&](shared_ptr<devices::HardwareDevice> device) {
 		return device->hardware_device()->driver() == driver; });
 
-	// Do the scan
-	auto devices = driver->scan(drvopts);
+	try {
+		// Do the scan
+		auto devices = driver->scan(drvopts);
 
-	// Add the scanned devices to the main list, set display names and sort.
-	for (shared_ptr<sigrok::HardwareDevice> device : devices) {
-		const shared_ptr<devices::HardwareDevice> d(
-			new devices::HardwareDevice(context_, device));
-		driver_devices.push_back(d);
+		// Add the scanned devices to the main list, set display names and sort.
+		for (shared_ptr<sigrok::HardwareDevice>& device : devices) {
+			const shared_ptr<devices::HardwareDevice> d(
+				new devices::HardwareDevice(context_, device));
+			driver_devices.push_back(d);
+		}
+
+		devices_.insert(devices_.end(), driver_devices.begin(),
+			driver_devices.end());
+		devices_.sort(bind(&DeviceManager::compare_devices, this, _1, _2));
+		driver_devices.sort(bind(
+			&DeviceManager::compare_devices, this, _1, _2));
+
+	} catch (const sigrok::Error &e) {
+		qWarning() << QApplication::tr("Error when scanning device driver '%1': %2").
+			arg(QString::fromStdString(driver->name()), e.what());
 	}
 
-	devices_.insert(devices_.end(), driver_devices.begin(),
-		driver_devices.end());
-	devices_.sort(bind(&DeviceManager::compare_devices, this, _1, _2));
-	driver_devices.sort(bind(
-		&DeviceManager::compare_devices, this, _1, _2));
-
 	return driver_devices;
 }
 
