@@ -97,7 +97,7 @@ bool StoreSession::start()
 	vector< shared_ptr<data::SignalBase> > achannel_list;
 	vector< shared_ptr<data::AnalogSegment> > asegment_list;
 
-	for (shared_ptr<data::SignalBase> signal : sigs) {
+	for (const shared_ptr<data::SignalBase>& signal : sigs) {
 		if (!signal->enabled())
 			continue;
 
@@ -145,6 +145,7 @@ bool StoreSession::start()
 	uint64_t end_sample;
 
 	if (sample_range_.first == sample_range_.second) {
+		// No sample range specified, save everything we have
 		start_sample_ = 0;
 		sample_count_ =	any_segment->get_sample_count();
 	} else {
@@ -159,6 +160,12 @@ bool StoreSession::start()
 		}
 	}
 
+	// Make sure the sample range is valid
+	if (start_sample_ > any_segment->get_sample_count()) {
+		error_ = tr("Can't save range without sample data.");
+		return false;
+	}
+
 	// Begin storing
 	try {
 		const auto context = session_.device_manager().context();
@@ -175,7 +182,7 @@ bool StoreSession::start()
 			{{ConfigKey::SAMPLERATE, Glib::Variant<guint64>::create(
 				any_segment->samplerate())}});
 		output_->receive(meta);
-	} catch (Error error) {
+	} catch (Error& error) {
 		error_ = tr("Error while saving: ") + error.what();
 		return false;
 	}
@@ -240,8 +247,8 @@ void StoreSession::store_proc(vector< shared_ptr<data::SignalBase> > achannel_li
 				shared_ptr<sigrok::Channel> achannel = (achannel_list.at(i))->channel();
 				shared_ptr<data::AnalogSegment> asegment = asegment_list.at(i);
 
-				const float *adata =
-					asegment->get_samples(start_sample_, start_sample_ + packet_len);
+				float *adata = new float[packet_len];
+				asegment->get_samples(start_sample_, start_sample_ + packet_len, adata);
 
 				auto analog = context->create_analog_packet(
 					vector<shared_ptr<sigrok::Channel> >{achannel},
@@ -257,11 +264,11 @@ void StoreSession::store_proc(vector< shared_ptr<data::SignalBase> > achannel_li
 			}
 
 			if (lsegment) {
-				const uint8_t* ldata =
-					lsegment->get_samples(start_sample_, start_sample_ + packet_len);
+				const size_t data_size = packet_len * lunit_size;
+				uint8_t* ldata = new uint8_t[data_size];
+				lsegment->get_samples(start_sample_, start_sample_ + packet_len, ldata);
 
-				const size_t length = packet_len * lunit_size;
-				auto logic = context->create_logic_packet((void*)ldata, length, lunit_size);
+				auto logic = context->create_logic_packet((void*)ldata, data_size, lunit_size);
 				const string ldata_str = output_->receive(logic);
 
 				if (output_stream_.is_open())
@@ -269,7 +276,7 @@ void StoreSession::store_proc(vector< shared_ptr<data::SignalBase> > achannel_li
 
 				delete[] ldata;
 			}
-		} catch (Error error) {
+		} catch (Error& error) {
 			error_ = tr("Error while saving: ") + error.what();
 			break;
 		}
