@@ -14,9 +14,16 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #define BOOST_TEST_MAIN
+#include "test/test.hpp"
 #include <boost/test/unit_test.hpp>
+
+using std::ostream;
+
+ostream& operator<<(ostream& stream, const QString& str)
+{
+	return stream << str.toUtf8().data();
+}
